<?php
/* --------------------------------------------------------------
   AutoUpdaterAjaxController.inc.php 2018-09-27
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2018 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

MainFactory::load_class('AdminHttpViewController');

/**
 * Class AutoUpdaterAjaxController
 */
class AutoUpdaterAjaxController extends AdminHttpViewController
{
	/**
	 * @var \AutoUpdater
	 */
	protected $autoUpdater;
	
	/**
	 * @var \AutoUpdaterFactory
	 */
	protected $autoUpdaterFactory;
	
	/**
	 * @var \CI_DB_query_builder
	 */
	protected $db;
	
	/**
	 * @var \LanguageTextManager
	 */
	private $textPhrases;
	
	/**
	 * @var string
	 */
	private $updateServer;
	
	/**
	 * @var \DataCache
	 */
	private $dataCache;
	
	/**
	 * @var \LogControl
	 */
	private $logControl;
	
	
	/**
	 * Initial method for this controller.
	 */
	public function init()
	{
		$this->db         = StaticGXCoreLoader::getDatabaseQueryBuilder();
		$this->dataCache  = DataCache::get_instance();
		$this->logControl = LogControl::get_instance();
		
		$this->autoUpdaterFactory = new AutoUpdaterFactory($this->db, $this->dataCache, $this->logControl);
		$this->autoUpdater        = $this->autoUpdaterFactory->createAutoUpdater();
		$this->textPhrases        = MainFactory::create('LanguageTextManager', 'auto_updater',
		                                                $_SESSION['languages_id']);
		$this->updateServer       = 'http://localhost/update-server/public/v2/callingHome.php';
	}
	
	
	/**
	 * Checks the permission for an update. Update ID must be given as post parameter.
	 *
	 * @return \JsonHttpControllerResponse
	 */
	public function actionCheckPermission()
	{
		try
		{
			$updateId = $this->_getPostData('updateId');
			$update   = $this->autoUpdater->fetchUpdateById($updateId);
			if($updateId === null || $update === null)
			{
				return MainFactory::create('JsonHttpControllerResponse', [
					'success' => false,
					'error'   => $this->textPhrases->get_text('ajax_error_no_update_selected'),
					'result'  => false,
					'list'    => []
				]);
			}
			
			$fileList = [];
			foreach($update->fileList() as $updateFile)
			{
				$fileList[] = DIR_FS_CATALOG . $updateFile['destination'];
			}
			
			return MainFactory::create('JsonHttpControllerResponse', [
				'success' => true,
				'error'   => '',
				'result'  => $this->autoUpdater->checkFilesPermissionsWithFileList($fileList),
				'list'    => $this->autoUpdater->getWrongPermittedFiles(),
			]);
		}
		catch(Exception $e)
		{
			return MainFactory::create('JsonHttpControllerResponse', [
				'success' => false,
				'error'   => $e->getMessage(),
				'result'  => false,
				'list'    => [],
			]);
		}
	}
	
	
	/**
	 * Checks the permission for an update. Update ID must be given as post parameter.
	 *
	 * @return \JsonHttpControllerResponse
	 */
	public function actionCheckPermissionForBackupRestore()
	{
		$backupId = $this->_getPostData('backupId');
		if($backupId === null)
		{
			return MainFactory::create('JsonHttpControllerResponse', [
				'success' => false,
				'error'   => $this->textPhrases->get_text('ajax_error_no_backup_selected'),
				'result'  => false,
				'list'    => []
			]);
		}
		
		return MainFactory::create('JsonHttpControllerResponse', [
			'success' => true,
			'error'   => '',
			'result'  => $this->autoUpdater->checkFilesPermissionsForBackup($backupId),
			'list'    => $this->autoUpdater->getWrongPermittedFiles(),
		]);
	}
	
	
	/**
	 * Deletes a backup by a given id.
	 *
	 * @return \JsonHttpControllerResponse
	 *
	 * @throws \AutoUpdaterBackupRestoreFailedException
	 */
	public function actionDeleteBackup()
	{
		$backupId = $this->_getPostData('backupId');
		if($backupId === null)
		{
			return MainFactory::create('JsonHttpControllerResponse', [
				'success' => false,
				'error'   => $this->textPhrases->get_text('ajax_error_no_backup_selected'),
			]);
		}
		
		$this->autoUpdater->deleteBackup($backupId);
		
		return MainFactory::create('JsonHttpControllerResponse', [
			'success' => true,
			'error'   => '',
		]);
	}
	
	
	/**
	 * Checks the ftp connection.
	 *
	 * @return \JsonHttpControllerResponse
	 */
	public function actionCheckFtpConnection()
	{
		$protocol = $this->_getPostData('ftp-protocol');
		$server   = $this->_getPostData('ftp-server');
		$login    = $this->_getPostData('ftp-login');
		$password = $this->_getPostData('ftp-password');
		$port     = $this->_getPostData('ftp-port');
		$passive  = $this->_getPostData('ftp-passive') === 'true';
		
		if(empty($protocol) || empty($server) || empty($login))
		{
			return MainFactory::create('JsonHttpControllerResponse', [
				'success' => false,
			]);
		}
		
		try
		{
			$this->autoUpdaterFactory->createFtpManager($protocol, $server, $login, $password, $port, $passive);
		}
		catch(AutoUpdaterException $e)
		{
			return MainFactory::create('JsonHttpControllerResponse', [
				'success' => false,
			]);
		}
		
		return MainFactory::create('JsonHttpControllerResponse', [
			'success' => true,
		]);
	}
	
	
	/**
	 * Creates a new debug log.
	 *
	 * @param string       $message
	 * @param string|array $additionalData
	 */
	protected function createDebugLog($message, $additionalData = '')
	{
		if(is_array($additionalData))
		{
			$additionalData = json_encode($additionalData);
		}
		
		$this->logControl->notice($message, '', 'auto_updater', 'notice', 'USER NOTICE', 0, $additionalData);
	}
}